# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.tax._registration_create_params import (
        RegistrationCreateParams,
    )
    from stripe.params.tax._registration_list_params import (
        RegistrationListParams,
    )
    from stripe.params.tax._registration_modify_params import (
        RegistrationModifyParams,
    )
    from stripe.params.tax._registration_retrieve_params import (
        RegistrationRetrieveParams,
    )


class Registration(
    CreateableAPIResource["Registration"],
    ListableAPIResource["Registration"],
    UpdateableAPIResource["Registration"],
):
    """
    A Tax `Registration` lets us know that your business is registered to collect tax on payments within a region, enabling you to [automatically collect tax](https://docs.stripe.com/tax).

    Stripe doesn't register on your behalf with the relevant authorities when you create a Tax `Registration` object. For more information on how to register to collect tax, see [our guide](https://docs.stripe.com/tax/registering).

    Related guide: [Using the Registrations API](https://docs.stripe.com/tax/registrations-api)
    """

    OBJECT_NAME: ClassVar[Literal["tax.registration"]] = "tax.registration"

    class CountryOptions(StripeObject):
        class Ae(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Al(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Am(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ao(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class At(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Au(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Aw(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Az(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ba(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Bb(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Bd(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Be(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Bf(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Bg(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Bh(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Bj(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Bs(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class By(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ca(StripeObject):
            class ProvinceStandard(StripeObject):
                province: str
                """
                Two-letter CA province code ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                """

            province_standard: Optional[ProvinceStandard]
            type: Literal["province_standard", "simplified", "standard"]
            """
            Type of registration in Canada.
            """
            _inner_class_types = {"province_standard": ProvinceStandard}

        class Cd(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Ch(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Cl(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Cm(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Co(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Cr(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Cv(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Cy(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Cz(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class De(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Dk(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Ec(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ee(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Eg(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Es(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Et(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Fi(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Fr(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Gb(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Ge(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Gn(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Gr(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Hr(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Hu(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Id(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ie(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class In(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Is(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class It(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Jp(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Ke(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Kg(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Kh(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Kr(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Kz(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class La(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Lk(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Lt(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Lu(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Lv(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Ma(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Md(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Me(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Mk(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Mr(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Mt(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Mx(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class My(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ng(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Nl(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class No(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Np(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Nz(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Om(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Pe(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ph(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Pl(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Pt(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Ro(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Rs(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Ru(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Sa(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Se(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Sg(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal["inbound_goods", "standard"]
                """
                Place of supply scheme used in an Default standard registration.
                """

            standard: Optional[Standard]
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """
            _inner_class_types = {"standard": Standard}

        class Si(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Sk(StripeObject):
            class Standard(StripeObject):
                place_of_supply_scheme: Literal[
                    "inbound_goods", "small_seller", "standard"
                ]
                """
                Place of supply scheme used in an EU standard registration.
                """

            standard: Optional[Standard]
            type: Literal["ioss", "oss_non_union", "oss_union", "standard"]
            """
            Type of registration in an EU country.
            """
            _inner_class_types = {"standard": Standard}

        class Sn(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Sr(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Th(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Tj(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Tr(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Tw(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Tz(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ua(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Ug(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Us(StripeObject):
            class AdmissionsTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=admissions_tax#registration-types) representing the local jurisdiction.
                """

            class AttendanceTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=attendance_tax#registration-types) representing the local jurisdiction.
                """

            class EntertainmentTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=entertainment_tax#registration-types) representing the local jurisdiction.
                """

            class GrossReceiptsTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=gross_receipts_tax#registration-types) representing the local jurisdiction.
                """

            class HomeRuleTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=home_rule_tax#registration-types) representing the local jurisdiction.
                """

            class HospitalityTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=hospitality_tax#registration-types) representing the local jurisdiction.
                """

            class LocalAmusementTax(StripeObject):
                jurisdiction: str
                """
                A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction.
                """

            class LocalLeaseTax(StripeObject):
                jurisdiction: str
                """
                A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction.
                """

            class LuxuryTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=luxury_tax#registration-types) representing the local jurisdiction.
                """

            class ResortTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=resort_tax#registration-types) representing the local jurisdiction.
                """

            class StateSalesTax(StripeObject):
                class Election(StripeObject):
                    jurisdiction: Optional[str]
                    """
                    A [FIPS code](https://www.census.gov/library/reference/code-lists/ansi.html) representing the local jurisdiction.
                    """
                    type: Literal[
                        "local_use_tax",
                        "simplified_sellers_use_tax",
                        "single_local_use_tax",
                    ]
                    """
                    The type of the election for the state sales tax registration.
                    """

                elections: Optional[List[Election]]
                """
                Elections for the state sales tax registration.
                """
                _inner_class_types = {"elections": Election}

            class TourismTax(StripeObject):
                jurisdiction: str
                """
                A [jurisdiction code](https://docs.stripe.com/tax/registering?type=tourism_tax#registration-types) representing the local jurisdiction.
                """

            admissions_tax: Optional[AdmissionsTax]
            attendance_tax: Optional[AttendanceTax]
            entertainment_tax: Optional[EntertainmentTax]
            gross_receipts_tax: Optional[GrossReceiptsTax]
            home_rule_tax: Optional[HomeRuleTax]
            hospitality_tax: Optional[HospitalityTax]
            local_amusement_tax: Optional[LocalAmusementTax]
            local_lease_tax: Optional[LocalLeaseTax]
            luxury_tax: Optional[LuxuryTax]
            resort_tax: Optional[ResortTax]
            state: str
            """
            Two-letter US state code ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """
            state_sales_tax: Optional[StateSalesTax]
            tourism_tax: Optional[TourismTax]
            type: Literal[
                "admissions_tax",
                "attendance_tax",
                "entertainment_tax",
                "gross_receipts_tax",
                "home_rule_tax",
                "hospitality_tax",
                "local_amusement_tax",
                "local_lease_tax",
                "luxury_tax",
                "resort_tax",
                "state_communications_tax",
                "state_retail_delivery_fee",
                "state_sales_tax",
                "tourism_tax",
            ]
            """
            Type of registration in the US.
            """
            _inner_class_types = {
                "admissions_tax": AdmissionsTax,
                "attendance_tax": AttendanceTax,
                "entertainment_tax": EntertainmentTax,
                "gross_receipts_tax": GrossReceiptsTax,
                "home_rule_tax": HomeRuleTax,
                "hospitality_tax": HospitalityTax,
                "local_amusement_tax": LocalAmusementTax,
                "local_lease_tax": LocalLeaseTax,
                "luxury_tax": LuxuryTax,
                "resort_tax": ResortTax,
                "state_sales_tax": StateSalesTax,
                "tourism_tax": TourismTax,
            }

        class Uy(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Uz(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Vn(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Za(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        class Zm(StripeObject):
            type: Literal["simplified"]
            """
            Type of registration in `country`.
            """

        class Zw(StripeObject):
            type: Literal["standard"]
            """
            Type of registration in `country`.
            """

        ae: Optional[Ae]
        al: Optional[Al]
        am: Optional[Am]
        ao: Optional[Ao]
        at: Optional[At]
        au: Optional[Au]
        aw: Optional[Aw]
        az: Optional[Az]
        ba: Optional[Ba]
        bb: Optional[Bb]
        bd: Optional[Bd]
        be: Optional[Be]
        bf: Optional[Bf]
        bg: Optional[Bg]
        bh: Optional[Bh]
        bj: Optional[Bj]
        bs: Optional[Bs]
        by: Optional[By]
        ca: Optional[Ca]
        cd: Optional[Cd]
        ch: Optional[Ch]
        cl: Optional[Cl]
        cm: Optional[Cm]
        co: Optional[Co]
        cr: Optional[Cr]
        cv: Optional[Cv]
        cy: Optional[Cy]
        cz: Optional[Cz]
        de: Optional[De]
        dk: Optional[Dk]
        ec: Optional[Ec]
        ee: Optional[Ee]
        eg: Optional[Eg]
        es: Optional[Es]
        et: Optional[Et]
        fi: Optional[Fi]
        fr: Optional[Fr]
        gb: Optional[Gb]
        ge: Optional[Ge]
        gn: Optional[Gn]
        gr: Optional[Gr]
        hr: Optional[Hr]
        hu: Optional[Hu]
        id: Optional[Id]
        ie: Optional[Ie]
        in_: Optional[In]
        is_: Optional[Is]
        it: Optional[It]
        jp: Optional[Jp]
        ke: Optional[Ke]
        kg: Optional[Kg]
        kh: Optional[Kh]
        kr: Optional[Kr]
        kz: Optional[Kz]
        la: Optional[La]
        lk: Optional[Lk]
        lt: Optional[Lt]
        lu: Optional[Lu]
        lv: Optional[Lv]
        ma: Optional[Ma]
        md: Optional[Md]
        me: Optional[Me]
        mk: Optional[Mk]
        mr: Optional[Mr]
        mt: Optional[Mt]
        mx: Optional[Mx]
        my: Optional[My]
        ng: Optional[Ng]
        nl: Optional[Nl]
        no: Optional[No]
        np: Optional[Np]
        nz: Optional[Nz]
        om: Optional[Om]
        pe: Optional[Pe]
        ph: Optional[Ph]
        pl: Optional[Pl]
        pt: Optional[Pt]
        ro: Optional[Ro]
        rs: Optional[Rs]
        ru: Optional[Ru]
        sa: Optional[Sa]
        se: Optional[Se]
        sg: Optional[Sg]
        si: Optional[Si]
        sk: Optional[Sk]
        sn: Optional[Sn]
        sr: Optional[Sr]
        th: Optional[Th]
        tj: Optional[Tj]
        tr: Optional[Tr]
        tw: Optional[Tw]
        tz: Optional[Tz]
        ua: Optional[Ua]
        ug: Optional[Ug]
        us: Optional[Us]
        uy: Optional[Uy]
        uz: Optional[Uz]
        vn: Optional[Vn]
        za: Optional[Za]
        zm: Optional[Zm]
        zw: Optional[Zw]
        _inner_class_types = {
            "ae": Ae,
            "al": Al,
            "am": Am,
            "ao": Ao,
            "at": At,
            "au": Au,
            "aw": Aw,
            "az": Az,
            "ba": Ba,
            "bb": Bb,
            "bd": Bd,
            "be": Be,
            "bf": Bf,
            "bg": Bg,
            "bh": Bh,
            "bj": Bj,
            "bs": Bs,
            "by": By,
            "ca": Ca,
            "cd": Cd,
            "ch": Ch,
            "cl": Cl,
            "cm": Cm,
            "co": Co,
            "cr": Cr,
            "cv": Cv,
            "cy": Cy,
            "cz": Cz,
            "de": De,
            "dk": Dk,
            "ec": Ec,
            "ee": Ee,
            "eg": Eg,
            "es": Es,
            "et": Et,
            "fi": Fi,
            "fr": Fr,
            "gb": Gb,
            "ge": Ge,
            "gn": Gn,
            "gr": Gr,
            "hr": Hr,
            "hu": Hu,
            "id": Id,
            "ie": Ie,
            "in": In,
            "is": Is,
            "it": It,
            "jp": Jp,
            "ke": Ke,
            "kg": Kg,
            "kh": Kh,
            "kr": Kr,
            "kz": Kz,
            "la": La,
            "lk": Lk,
            "lt": Lt,
            "lu": Lu,
            "lv": Lv,
            "ma": Ma,
            "md": Md,
            "me": Me,
            "mk": Mk,
            "mr": Mr,
            "mt": Mt,
            "mx": Mx,
            "my": My,
            "ng": Ng,
            "nl": Nl,
            "no": No,
            "np": Np,
            "nz": Nz,
            "om": Om,
            "pe": Pe,
            "ph": Ph,
            "pl": Pl,
            "pt": Pt,
            "ro": Ro,
            "rs": Rs,
            "ru": Ru,
            "sa": Sa,
            "se": Se,
            "sg": Sg,
            "si": Si,
            "sk": Sk,
            "sn": Sn,
            "sr": Sr,
            "th": Th,
            "tj": Tj,
            "tr": Tr,
            "tw": Tw,
            "tz": Tz,
            "ua": Ua,
            "ug": Ug,
            "us": Us,
            "uy": Uy,
            "uz": Uz,
            "vn": Vn,
            "za": Za,
            "zm": Zm,
            "zw": Zw,
        }
        _field_remappings = {"in_": "in", "is_": "is"}

    active_from: int
    """
    Time at which the registration becomes active. Measured in seconds since the Unix epoch.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    country_options: CountryOptions
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    expires_at: Optional[int]
    """
    If set, the registration stops being active at this time. If not set, the registration will be active indefinitely. Measured in seconds since the Unix epoch.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["tax.registration"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Literal["active", "expired", "scheduled"]
    """
    The status of the registration. This field is present for convenience and can be deduced from `active_from` and `expires_at`.
    """

    @classmethod
    def create(
        cls, **params: Unpack["RegistrationCreateParams"]
    ) -> "Registration":
        """
        Creates a new Tax Registration object.
        """
        return cast(
            "Registration",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["RegistrationCreateParams"]
    ) -> "Registration":
        """
        Creates a new Tax Registration object.
        """
        return cast(
            "Registration",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["RegistrationListParams"]
    ) -> ListObject["Registration"]:
        """
        Returns a list of Tax Registration objects.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["RegistrationListParams"]
    ) -> ListObject["Registration"]:
        """
        Returns a list of Tax Registration objects.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["RegistrationModifyParams"]
    ) -> "Registration":
        """
        Updates an existing Tax Registration object.

        A registration cannot be deleted after it has been created. If you wish to end a registration you may do so by setting expires_at.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Registration",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["RegistrationModifyParams"]
    ) -> "Registration":
        """
        Updates an existing Tax Registration object.

        A registration cannot be deleted after it has been created. If you wish to end a registration you may do so by setting expires_at.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Registration",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["RegistrationRetrieveParams"]
    ) -> "Registration":
        """
        Returns a Tax Registration object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["RegistrationRetrieveParams"]
    ) -> "Registration":
        """
        Returns a Tax Registration object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"country_options": CountryOptions}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._price import Price
    from stripe._request_options import RequestOptions
    from stripe._search_result_object import SearchResultObject
    from stripe.params._price_create_params import PriceCreateParams
    from stripe.params._price_list_params import PriceListParams
    from stripe.params._price_retrieve_params import PriceRetrieveParams
    from stripe.params._price_search_params import PriceSearchParams
    from stripe.params._price_update_params import PriceUpdateParams


class PriceService(StripeService):
    def list(
        self,
        params: Optional["PriceListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Price]":
        """
        Returns a list of your active prices, excluding [inline prices](https://docs.stripe.com/docs/products-prices/pricing-models#inline-pricing). For the list of inactive prices, set active to false.
        """
        return cast(
            "ListObject[Price]",
            self._request(
                "get",
                "/v1/prices",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PriceListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Price]":
        """
        Returns a list of your active prices, excluding [inline prices](https://docs.stripe.com/docs/products-prices/pricing-models#inline-pricing). For the list of inactive prices, set active to false.
        """
        return cast(
            "ListObject[Price]",
            await self._request_async(
                "get",
                "/v1/prices",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "PriceCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Price":
        """
        Creates a new [Price for an existing <a href="https://docs.stripe.com/api/products">Product](https://docs.stripe.com/api/prices). The Price can be recurring or one-time.
        """
        return cast(
            "Price",
            self._request(
                "post",
                "/v1/prices",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PriceCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Price":
        """
        Creates a new [Price for an existing <a href="https://docs.stripe.com/api/products">Product](https://docs.stripe.com/api/prices). The Price can be recurring or one-time.
        """
        return cast(
            "Price",
            await self._request_async(
                "post",
                "/v1/prices",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        price: str,
        params: Optional["PriceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Price":
        """
        Retrieves the price with the given ID.
        """
        return cast(
            "Price",
            self._request(
                "get",
                "/v1/prices/{price}".format(price=sanitize_id(price)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        price: str,
        params: Optional["PriceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Price":
        """
        Retrieves the price with the given ID.
        """
        return cast(
            "Price",
            await self._request_async(
                "get",
                "/v1/prices/{price}".format(price=sanitize_id(price)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        price: str,
        params: Optional["PriceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Price":
        """
        Updates the specified price by setting the values of the parameters passed. Any parameters not provided are left unchanged.
        """
        return cast(
            "Price",
            self._request(
                "post",
                "/v1/prices/{price}".format(price=sanitize_id(price)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        price: str,
        params: Optional["PriceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Price":
        """
        Updates the specified price by setting the values of the parameters passed. Any parameters not provided are left unchanged.
        """
        return cast(
            "Price",
            await self._request_async(
                "post",
                "/v1/prices/{price}".format(price=sanitize_id(price)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def search(
        self,
        params: "PriceSearchParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SearchResultObject[Price]":
        """
        Search for prices you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            "SearchResultObject[Price]",
            self._request(
                "get",
                "/v1/prices/search",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def search_async(
        self,
        params: "PriceSearchParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SearchResultObject[Price]":
        """
        Search for prices you've previously created using Stripe's [Search Query Language](https://docs.stripe.com/docs/search#search-query-language).
        Don't use search in read-after-write flows where strict consistency is necessary. Under normal operating
        conditions, data is searchable in less than a minute. Occasionally, propagation of new or updated data can be up
        to an hour behind during outages. Search functionality is not available to merchants in India.
        """
        return cast(
            "SearchResultObject[Price]",
            await self._request_async(
                "get",
                "/v1/prices/search",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_create(
        self,
        params: Optional["PriceCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Price create request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_update(
        self,
        price: str,
        params: Optional["PriceUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a Price update request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"price": price},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class BalanceSettingsUpdateParams(TypedDict):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    payments: NotRequired["BalanceSettingsUpdateParamsPayments"]
    """
    Settings that apply to the [Payments Balance](https://docs.stripe.com/api/balance).
    """


class BalanceSettingsUpdateParamsPayments(TypedDict):
    debit_negative_balances: NotRequired[bool]
    """
    A Boolean indicating whether Stripe should try to reclaim negative balances from an attached bank account. For details, see [Understanding Connect Account Balances](https://docs.stripe.com/connect/account-balances).
    """
    payouts: NotRequired["BalanceSettingsUpdateParamsPaymentsPayouts"]
    """
    Settings specific to the account's payouts.
    """
    settlement_currencies: NotRequired[
        "Dict[str, Literal['disabled', 'enabled']]|UntypedStripeObject[Literal['disabled', 'enabled']]"
    ]
    """
    A hash of settlement currencies to update. Each key is an ISO 4217 currency code, and the value is either `enabled` or `disabled`.
    """
    settlement_timing: NotRequired[
        "BalanceSettingsUpdateParamsPaymentsSettlementTiming"
    ]
    """
    Settings related to the account's balance settlement timing.
    """


class BalanceSettingsUpdateParamsPaymentsPayouts(TypedDict):
    automatic_transfer_rules_by_currency: NotRequired[
        "Literal['']|Dict[str, Union[Literal[''], List[BalanceSettingsUpdateParamsPaymentsPayoutsAutomaticTransferRulesByCurrency]]]|UntypedStripeObject[Union[Literal[''], List[BalanceSettingsUpdateParamsPaymentsPayoutsAutomaticTransferRulesByCurrency]]]"
    ]
    """
    Configures per-currency rules for automatically transferring funds from the payments balance to a FinancialAccount.
    """
    minimum_balance_by_currency: NotRequired[
        "Literal['']|Dict[str, Union[Literal[''], int]]|UntypedStripeObject[Union[Literal[''], int]]"
    ]
    """
    The minimum balance amount to retain per currency after automatic payouts. Only funds that exceed these amounts are paid out. Learn more about the [minimum balances for automatic payouts](https://docs.stripe.com/payouts/minimum-balances-for-automatic-payouts).
    """
    schedule: NotRequired["BalanceSettingsUpdateParamsPaymentsPayoutsSchedule"]
    """
    Details on when funds from charges are available, and when they are paid out to an external account. For details, see our [Setting Bank and Debit Card Payouts](https://docs.stripe.com/connect/bank-transfers#payout-information) documentation.
    """
    statement_descriptor: NotRequired[str]
    """
    The text that appears on the bank account statement for payouts. If not set, this defaults to the platform's bank descriptor as set in the Dashboard.
    """


class BalanceSettingsUpdateParamsPaymentsPayoutsAutomaticTransferRulesByCurrency(
    TypedDict,
):
    payout_method: str
    """
    The ID of the FinancialAccount that funds will be transferred to during automatic transfers.
    """
    transfer_up_to_amount: NotRequired[int]
    """
    The maximum amount in minor units to transfer to the FinancialAccount. Required and only applicable when `type` is `transfer_up_to_amount`.
    """
    type: Literal["transfer_all", "transfer_up_to_amount"]
    """
    The type of automatic transfer rule.
    """


class BalanceSettingsUpdateParamsPaymentsPayoutsSchedule(TypedDict):
    interval: NotRequired[Literal["daily", "manual", "monthly", "weekly"]]
    """
    How frequently available funds are paid out. One of: `daily`, `manual`, `weekly`, or `monthly`. Default is `daily`.
    """
    monthly_payout_days: NotRequired[List[int]]
    """
    The days of the month when available funds are paid out, specified as an array of numbers between 1--31. Payouts nominally scheduled between the 29th and 31st of the month are instead sent on the last day of a shorter month. Required and applicable only if `interval` is `monthly`.
    """
    weekly_payout_days: NotRequired[
        List[Literal["friday", "monday", "thursday", "tuesday", "wednesday"]]
    ]
    """
    The days of the week when available funds are paid out, specified as an array, e.g., [`monday`, `tuesday`]. Required and applicable only if `interval` is `weekly`.
    """


class BalanceSettingsUpdateParamsPaymentsSettlementTiming(TypedDict):
    delay_days_override: NotRequired["Literal['']|int"]
    """
    Change `delay_days` for this account, which determines the number of days charge funds are held before becoming available. The maximum value is 31. Passing an empty string to `delay_days_override` will return `delay_days` to the default, which is the lowest available value for the account. [Learn more about controlling delay days](https://docs.stripe.com/connect/manage-payout-schedule).
    """
    start_of_day: NotRequired[
        "Literal['']|BalanceSettingsUpdateParamsPaymentsSettlementTimingStartOfDay"
    ]
    """
    Customized start of day configuration for automatic payouts to group and send payments in local timezones with a customized day starting time. For details, see our [Customized start of day](https://docs.stripe.com/connect/customized-start-of-day) documentation.
    """


class BalanceSettingsUpdateParamsPaymentsSettlementTimingStartOfDay(TypedDict):
    hour: NotRequired[int]
    """
    Hour at which the customized start of day begins according to the given timezone. Must be a [supported customized start of day hour](https://docs.stripe.com/connect/customized-start-of-day#available-timezones-and-cutoffs).
    """
    minutes: NotRequired[int]
    """
    Minutes at which the customized start of day begins according to the given timezone. Must be either 0 or 30.
    """
    timezone: NotRequired[str]
    """
    Timezone for the customized start of day. Must be a [supported customized start of day timezone](https://docs.stripe.com/connect/customized-start-of-day#available-timezones-and-cutoffs).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import NotRequired


class MeterEventCreateParams(RequestOptions):
    event_name: str
    """
    The name of the meter event. Corresponds with the `event_name` field on a meter.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    identifier: NotRequired[str]
    """
    A unique identifier for the event. If not provided, one is generated. We recommend using UUID-like identifiers. Stripe enforces uniqueness within a rolling period of at least 24 hours. The enforcement of uniqueness primarily addresses issues arising from accidental retries or other problems occurring within extremely brief time intervals. This approach helps prevent duplicate entries and ensures data integrity in high-frequency operations.
    """
    payload: "Dict[str, str]|UntypedStripeObject[str]"
    """
    The payload of the event. This must contain the fields corresponding to a meter's `customer_mapping.event_payload_key` (default is `stripe_customer_id`) and `value_settings.event_payload_key` (default is `value`). Read more about the [payload](https://docs.stripe.com/billing/subscriptions/usage-based/meters/configure#meter-configuration-attributes).
    """
    timestamp: NotRequired[int]
    """
    The time of the event. Measured in seconds since the Unix epoch. Must be within the past 35 calendar days or up to 5 minutes in the future. Defaults to current timestamp if not specified.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class WebhookEndpointUpdateParams(TypedDict):
    description: NotRequired["Literal['']|str"]
    """
    An optional description of what the webhook is used for.
    """
    disabled: NotRequired[bool]
    """
    Disable the webhook endpoint if set to true.
    """
    enabled_events: NotRequired[
        List[
            Literal[
                "*",
                "account.application.authorized",
                "account.application.deauthorized",
                "account.external_account.created",
                "account.external_account.deleted",
                "account.external_account.updated",
                "account.updated",
                "account_notice.created",
                "account_notice.updated",
                "application_fee.created",
                "application_fee.refund.updated",
                "application_fee.refunded",
                "balance.available",
                "balance_settings.updated",
                "billing.alert.triggered",
                "billing.credit_balance_transaction.created",
                "billing.credit_grant.created",
                "billing.credit_grant.updated",
                "billing.meter.created",
                "billing.meter.deactivated",
                "billing.meter.reactivated",
                "billing.meter.updated",
                "billing_portal.configuration.created",
                "billing_portal.configuration.updated",
                "billing_portal.session.created",
                "capability.updated",
                "capital.financing_offer.accepted",
                "capital.financing_offer.accepted_other_offer",
                "capital.financing_offer.canceled",
                "capital.financing_offer.created",
                "capital.financing_offer.expired",
                "capital.financing_offer.fully_repaid",
                "capital.financing_offer.paid_out",
                "capital.financing_offer.rejected",
                "capital.financing_offer.replacement_created",
                "capital.financing_summary.line_of_credit_update",
                "capital.financing_transaction.created",
                "cash_balance.funds_available",
                "charge.captured",
                "charge.dispute.closed",
                "charge.dispute.created",
                "charge.dispute.funds_reinstated",
                "charge.dispute.funds_withdrawn",
                "charge.dispute.updated",
                "charge.expired",
                "charge.failed",
                "charge.pending",
                "charge.refund.updated",
                "charge.refunded",
                "charge.succeeded",
                "charge.updated",
                "checkout.session.async_payment_failed",
                "checkout.session.async_payment_succeeded",
                "checkout.session.completed",
                "checkout.session.expired",
                "climate.order.canceled",
                "climate.order.created",
                "climate.order.delayed",
                "climate.order.delivered",
                "climate.order.product_substituted",
                "climate.product.created",
                "climate.product.pricing_updated",
                "coupon.created",
                "coupon.deleted",
                "coupon.updated",
                "credit_note.created",
                "credit_note.updated",
                "credit_note.voided",
                "customer.created",
                "customer.deleted",
                "customer.discount.created",
                "customer.discount.deleted",
                "customer.discount.updated",
                "customer.source.created",
                "customer.source.deleted",
                "customer.source.expiring",
                "customer.source.updated",
                "customer.subscription.collection_paused",
                "customer.subscription.collection_resumed",
                "customer.subscription.created",
                "customer.subscription.custom_event",
                "customer.subscription.deleted",
                "customer.subscription.paused",
                "customer.subscription.pending_update_applied",
                "customer.subscription.pending_update_expired",
                "customer.subscription.price_migration_failed",
                "customer.subscription.resumed",
                "customer.subscription.trial_will_end",
                "customer.subscription.updated",
                "customer.tax_id.created",
                "customer.tax_id.deleted",
                "customer.tax_id.updated",
                "customer.updated",
                "customer_cash_balance_transaction.created",
                "entitlements.active_entitlement_summary.updated",
                "file.created",
                "financial_connections.account.account_numbers_updated",
                "financial_connections.account.created",
                "financial_connections.account.deactivated",
                "financial_connections.account.disconnected",
                "financial_connections.account.reactivated",
                "financial_connections.account.refreshed_balance",
                "financial_connections.account.refreshed_inferred_balances",
                "financial_connections.account.refreshed_ownership",
                "financial_connections.account.refreshed_transactions",
                "financial_connections.account.upcoming_account_number_expiry",
                "financial_connections.session.updated",
                "fx_quote.expired",
                "identity.verification_session.canceled",
                "identity.verification_session.created",
                "identity.verification_session.processing",
                "identity.verification_session.redacted",
                "identity.verification_session.requires_input",
                "identity.verification_session.verified",
                "invoice.created",
                "invoice.deleted",
                "invoice.finalization_failed",
                "invoice.finalized",
                "invoice.marked_uncollectible",
                "invoice.overdue",
                "invoice.overpaid",
                "invoice.paid",
                "invoice.payment.overpaid",
                "invoice.payment_action_required",
                "invoice.payment_attempt_required",
                "invoice.payment_failed",
                "invoice.payment_succeeded",
                "invoice.sent",
                "invoice.upcoming",
                "invoice.updated",
                "invoice.voided",
                "invoice.will_be_due",
                "invoice_payment.paid",
                "invoiceitem.created",
                "invoiceitem.deleted",
                "issuing_authorization.created",
                "issuing_authorization.request",
                "issuing_authorization.updated",
                "issuing_card.created",
                "issuing_card.updated",
                "issuing_cardholder.created",
                "issuing_cardholder.updated",
                "issuing_dispute.closed",
                "issuing_dispute.created",
                "issuing_dispute.funds_reinstated",
                "issuing_dispute.funds_rescinded",
                "issuing_dispute.submitted",
                "issuing_dispute.updated",
                "issuing_dispute_settlement_detail.created",
                "issuing_dispute_settlement_detail.updated",
                "issuing_fraud_liability_debit.created",
                "issuing_personalization_design.activated",
                "issuing_personalization_design.deactivated",
                "issuing_personalization_design.rejected",
                "issuing_personalization_design.updated",
                "issuing_settlement.created",
                "issuing_settlement.updated",
                "issuing_token.created",
                "issuing_token.updated",
                "issuing_transaction.created",
                "issuing_transaction.purchase_details_receipt_updated",
                "issuing_transaction.updated",
                "mandate.updated",
                "payment_intent.amount_capturable_updated",
                "payment_intent.canceled",
                "payment_intent.created",
                "payment_intent.partially_funded",
                "payment_intent.payment_failed",
                "payment_intent.processing",
                "payment_intent.requires_action",
                "payment_intent.succeeded",
                "payment_link.created",
                "payment_link.updated",
                "payment_method.attached",
                "payment_method.automatically_updated",
                "payment_method.detached",
                "payment_method.updated",
                "payout.canceled",
                "payout.created",
                "payout.failed",
                "payout.paid",
                "payout.reconciliation_completed",
                "payout.updated",
                "person.created",
                "person.deleted",
                "person.updated",
                "plan.created",
                "plan.deleted",
                "plan.updated",
                "price.created",
                "price.deleted",
                "price.updated",
                "privacy.redaction_job.canceled",
                "privacy.redaction_job.created",
                "privacy.redaction_job.ready",
                "privacy.redaction_job.succeeded",
                "privacy.redaction_job.validation_error",
                "product.created",
                "product.deleted",
                "product.updated",
                "promotion_code.created",
                "promotion_code.updated",
                "quote.accept_failed",
                "quote.accepted",
                "quote.accepting",
                "quote.canceled",
                "quote.created",
                "quote.draft",
                "quote.finalized",
                "quote.reestimate_failed",
                "quote.reestimated",
                "quote.stale",
                "radar.early_fraud_warning.created",
                "radar.early_fraud_warning.updated",
                "refund.created",
                "refund.failed",
                "refund.updated",
                "reporting.report_run.failed",
                "reporting.report_run.succeeded",
                "reporting.report_type.updated",
                "reserve.hold.created",
                "reserve.hold.updated",
                "reserve.plan.created",
                "reserve.plan.disabled",
                "reserve.plan.expired",
                "reserve.plan.updated",
                "reserve.release.created",
                "review.closed",
                "review.opened",
                "setup_intent.canceled",
                "setup_intent.created",
                "setup_intent.requires_action",
                "setup_intent.setup_failed",
                "setup_intent.succeeded",
                "sigma.scheduled_query_run.created",
                "source.canceled",
                "source.chargeable",
                "source.failed",
                "source.mandate_notification",
                "source.refund_attributes_required",
                "source.transaction.created",
                "source.transaction.updated",
                "subscription_schedule.aborted",
                "subscription_schedule.canceled",
                "subscription_schedule.completed",
                "subscription_schedule.created",
                "subscription_schedule.expiring",
                "subscription_schedule.price_migration_failed",
                "subscription_schedule.released",
                "subscription_schedule.updated",
                "tax.form.updated",
                "tax.settings.updated",
                "tax_rate.created",
                "tax_rate.updated",
                "terminal.reader.action_failed",
                "terminal.reader.action_succeeded",
                "terminal.reader.action_updated",
                "test_helpers.test_clock.advancing",
                "test_helpers.test_clock.created",
                "test_helpers.test_clock.deleted",
                "test_helpers.test_clock.internal_failure",
                "test_helpers.test_clock.ready",
                "topup.canceled",
                "topup.created",
                "topup.failed",
                "topup.reversed",
                "topup.succeeded",
                "transfer.created",
                "transfer.reversed",
                "transfer.updated",
                "treasury.credit_reversal.created",
                "treasury.credit_reversal.posted",
                "treasury.debit_reversal.completed",
                "treasury.debit_reversal.created",
                "treasury.debit_reversal.initial_credit_granted",
                "treasury.financial_account.closed",
                "treasury.financial_account.created",
                "treasury.financial_account.features_status_updated",
                "treasury.inbound_transfer.canceled",
                "treasury.inbound_transfer.created",
                "treasury.inbound_transfer.failed",
                "treasury.inbound_transfer.succeeded",
                "treasury.outbound_payment.canceled",
                "treasury.outbound_payment.created",
                "treasury.outbound_payment.expected_arrival_date_updated",
                "treasury.outbound_payment.failed",
                "treasury.outbound_payment.posted",
                "treasury.outbound_payment.returned",
                "treasury.outbound_payment.tracking_details_updated",
                "treasury.outbound_transfer.canceled",
                "treasury.outbound_transfer.created",
                "treasury.outbound_transfer.expected_arrival_date_updated",
                "treasury.outbound_transfer.failed",
                "treasury.outbound_transfer.posted",
                "treasury.outbound_transfer.returned",
                "treasury.outbound_transfer.tracking_details_updated",
                "treasury.received_credit.created",
                "treasury.received_credit.failed",
                "treasury.received_credit.succeeded",
                "treasury.received_debit.created",
            ]
        ]
    ]
    """
    The list of events to enable for this endpoint. You may specify `['*']` to enable all events, except those that require explicit selection.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    url: NotRequired[str]
    """
    The URL of the webhook endpoint.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class RequestedSessionConfirmParams(RequestOptions):
    affiliate_attribution: NotRequired[
        "RequestedSessionConfirmParamsAffiliateAttribution"
    ]
    """
    Affiliate attribution data associated with this requested session.
    """
    buyer_consents: NotRequired["RequestedSessionConfirmParamsBuyerConsents"]
    """
    The buyer's consent choices for marketing communications.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    The metadata for this requested session.
    """
    payment_method: NotRequired[str]
    """
    The PaymentMethod to use with the requested session.
    """
    return_url: NotRequired[str]
    """
    The URL to redirect your customer back to after they authenticate or complete a payment action. Required for redirect-based payment methods such as Affirm or Klarna.
    """
    risk_details: NotRequired["RequestedSessionConfirmParamsRiskDetails"]
    """
    Risk details/signals associated with the requested session
    """


class RequestedSessionConfirmParamsAffiliateAttribution(TypedDict):
    campaign_id: NotRequired[str]
    """
    Agent-scoped campaign identifier.
    """
    creative_id: NotRequired[str]
    """
    Agent-scoped creative identifier.
    """
    expires_at: int
    """
    Timestamp when the attribution token expires.
    """
    identification_token: str
    """
    Agent-issued secret to validate the legitimacy of the source of this data.
    """
    issued_at: int
    """
    Timestamp for when the attribution token was issued.
    """
    provider: str
    """
    Identifier for the attribution agent / affiliate network namespace.
    """
    publisher_id: NotRequired[str]
    """
    Agent-scoped affiliate/publisher identifier.
    """
    shared_metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Freeform key/value pairs for additional non-sensitive per-agent data.
    """
    source: NotRequired[
        "RequestedSessionConfirmParamsAffiliateAttributionSource"
    ]
    """
    Context about where the attribution originated.
    """
    sub_id: NotRequired[str]
    """
    Agent-scoped sub-tracking identifier.
    """
    touchpoint: Literal["first", "last"]
    """
    Whether this is the first or last touchpoint.
    """


class RequestedSessionConfirmParamsAffiliateAttributionSource(TypedDict):
    platform: NotRequired[str]
    """
    The platform where the attribution originated.
    """
    type: Literal["platform", "url"]
    """
    The type of the attribution source.
    """
    url: NotRequired[str]
    """
    The URL where the attribution originated.
    """


class RequestedSessionConfirmParamsBuyerConsents(TypedDict):
    marketing: NotRequired[
        "RequestedSessionConfirmParamsBuyerConsentsMarketing"
    ]
    """
    The marketing consent data for the buyer.
    """


class RequestedSessionConfirmParamsBuyerConsentsMarketing(TypedDict):
    consents: NotRequired[
        List["RequestedSessionConfirmParamsBuyerConsentsMarketingConsent"]
    ]
    """
    The list of marketing consent entries.
    """


class RequestedSessionConfirmParamsBuyerConsentsMarketingConsent(TypedDict):
    channel: Literal["email", "sms"]
    """
    The marketing consent channel.
    """
    status: Literal["granted", "none"]
    """
    The consent status. Use 'granted' to indicate the buyer has opted in.
    """


class RequestedSessionConfirmParamsRiskDetails(TypedDict):
    client_device_metadata_details: NotRequired[
        "RequestedSessionConfirmParamsRiskDetailsClientDeviceMetadataDetails"
    ]
    """
    The client device metadata details for this requested session.
    """


class RequestedSessionConfirmParamsRiskDetailsClientDeviceMetadataDetails(
    TypedDict,
):
    radar_session: NotRequired[str]
    """
    The radar session.
    """
    referrer: NotRequired[str]
    """
    The referrer of the client device.
    """
    remote_ip: NotRequired[str]
    """
    The remote IP address of the client device.
    """
    time_on_page_ms: NotRequired[int]
    """
    The time on page in milliseconds.
    """
    user_agent: NotRequired[str]
    """
    The user agent of the client device.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.payments import (
        settlement_allocation_intents as settlement_allocation_intents,
    )
    from stripe.params.v2.payments._off_session_payment_cancel_params import (
        OffSessionPaymentCancelParams as OffSessionPaymentCancelParams,
    )
    from stripe.params.v2.payments._off_session_payment_capture_params import (
        OffSessionPaymentCaptureParams as OffSessionPaymentCaptureParams,
        OffSessionPaymentCaptureParamsAmountDetails as OffSessionPaymentCaptureParamsAmountDetails,
        OffSessionPaymentCaptureParamsAmountDetailsLineItem as OffSessionPaymentCaptureParamsAmountDetailsLineItem,
        OffSessionPaymentCaptureParamsAmountDetailsLineItemTax as OffSessionPaymentCaptureParamsAmountDetailsLineItemTax,
        OffSessionPaymentCaptureParamsAmountDetailsShipping as OffSessionPaymentCaptureParamsAmountDetailsShipping,
        OffSessionPaymentCaptureParamsAmountDetailsTax as OffSessionPaymentCaptureParamsAmountDetailsTax,
        OffSessionPaymentCaptureParamsPaymentDetails as OffSessionPaymentCaptureParamsPaymentDetails,
        OffSessionPaymentCaptureParamsTransferData as OffSessionPaymentCaptureParamsTransferData,
    )
    from stripe.params.v2.payments._off_session_payment_create_params import (
        OffSessionPaymentCreateParams as OffSessionPaymentCreateParams,
        OffSessionPaymentCreateParamsAmountDetails as OffSessionPaymentCreateParamsAmountDetails,
        OffSessionPaymentCreateParamsAmountDetailsLineItem as OffSessionPaymentCreateParamsAmountDetailsLineItem,
        OffSessionPaymentCreateParamsAmountDetailsLineItemTax as OffSessionPaymentCreateParamsAmountDetailsLineItemTax,
        OffSessionPaymentCreateParamsAmountDetailsShipping as OffSessionPaymentCreateParamsAmountDetailsShipping,
        OffSessionPaymentCreateParamsAmountDetailsTax as OffSessionPaymentCreateParamsAmountDetailsTax,
        OffSessionPaymentCreateParamsCapture as OffSessionPaymentCreateParamsCapture,
        OffSessionPaymentCreateParamsPaymentDetails as OffSessionPaymentCreateParamsPaymentDetails,
        OffSessionPaymentCreateParamsPaymentMethodData as OffSessionPaymentCreateParamsPaymentMethodData,
        OffSessionPaymentCreateParamsPaymentMethodDataBillingDetails as OffSessionPaymentCreateParamsPaymentMethodDataBillingDetails,
        OffSessionPaymentCreateParamsPaymentMethodDataBillingDetailsAddress as OffSessionPaymentCreateParamsPaymentMethodDataBillingDetailsAddress,
        OffSessionPaymentCreateParamsPaymentMethodDataCard as OffSessionPaymentCreateParamsPaymentMethodDataCard,
        OffSessionPaymentCreateParamsPaymentMethodOptions as OffSessionPaymentCreateParamsPaymentMethodOptions,
        OffSessionPaymentCreateParamsPaymentMethodOptionsCard as OffSessionPaymentCreateParamsPaymentMethodOptionsCard,
        OffSessionPaymentCreateParamsPaymentsOrchestration as OffSessionPaymentCreateParamsPaymentsOrchestration,
        OffSessionPaymentCreateParamsRetryDetails as OffSessionPaymentCreateParamsRetryDetails,
        OffSessionPaymentCreateParamsTransferData as OffSessionPaymentCreateParamsTransferData,
    )
    from stripe.params.v2.payments._off_session_payment_list_params import (
        OffSessionPaymentListParams as OffSessionPaymentListParams,
    )
    from stripe.params.v2.payments._off_session_payment_pause_params import (
        OffSessionPaymentPauseParams as OffSessionPaymentPauseParams,
    )
    from stripe.params.v2.payments._off_session_payment_resume_params import (
        OffSessionPaymentResumeParams as OffSessionPaymentResumeParams,
    )
    from stripe.params.v2.payments._off_session_payment_retrieve_params import (
        OffSessionPaymentRetrieveParams as OffSessionPaymentRetrieveParams,
    )
    from stripe.params.v2.payments._settlement_allocation_intent_cancel_params import (
        SettlementAllocationIntentCancelParams as SettlementAllocationIntentCancelParams,
    )
    from stripe.params.v2.payments._settlement_allocation_intent_create_params import (
        SettlementAllocationIntentCreateParams as SettlementAllocationIntentCreateParams,
    )
    from stripe.params.v2.payments._settlement_allocation_intent_list_params import (
        SettlementAllocationIntentListParams as SettlementAllocationIntentListParams,
    )
    from stripe.params.v2.payments._settlement_allocation_intent_retrieve_params import (
        SettlementAllocationIntentRetrieveParams as SettlementAllocationIntentRetrieveParams,
    )
    from stripe.params.v2.payments._settlement_allocation_intent_submit_params import (
        SettlementAllocationIntentSubmitParams as SettlementAllocationIntentSubmitParams,
    )
    from stripe.params.v2.payments._settlement_allocation_intent_update_params import (
        SettlementAllocationIntentUpdateParams as SettlementAllocationIntentUpdateParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "settlement_allocation_intents": (
        "stripe.params.v2.payments.settlement_allocation_intents",
        True,
    ),
    "OffSessionPaymentCancelParams": (
        "stripe.params.v2.payments._off_session_payment_cancel_params",
        False,
    ),
    "OffSessionPaymentCaptureParams": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsAmountDetails": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsAmountDetailsLineItem": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsAmountDetailsLineItemTax": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsAmountDetailsShipping": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsAmountDetailsTax": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsPaymentDetails": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCaptureParamsTransferData": (
        "stripe.params.v2.payments._off_session_payment_capture_params",
        False,
    ),
    "OffSessionPaymentCreateParams": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsAmountDetails": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsAmountDetailsLineItem": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsAmountDetailsLineItemTax": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsAmountDetailsShipping": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsAmountDetailsTax": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsCapture": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentDetails": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentMethodData": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentMethodDataBillingDetails": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentMethodDataBillingDetailsAddress": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentMethodDataCard": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentMethodOptions": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentMethodOptionsCard": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsPaymentsOrchestration": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsRetryDetails": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentCreateParamsTransferData": (
        "stripe.params.v2.payments._off_session_payment_create_params",
        False,
    ),
    "OffSessionPaymentListParams": (
        "stripe.params.v2.payments._off_session_payment_list_params",
        False,
    ),
    "OffSessionPaymentPauseParams": (
        "stripe.params.v2.payments._off_session_payment_pause_params",
        False,
    ),
    "OffSessionPaymentResumeParams": (
        "stripe.params.v2.payments._off_session_payment_resume_params",
        False,
    ),
    "OffSessionPaymentRetrieveParams": (
        "stripe.params.v2.payments._off_session_payment_retrieve_params",
        False,
    ),
    "SettlementAllocationIntentCancelParams": (
        "stripe.params.v2.payments._settlement_allocation_intent_cancel_params",
        False,
    ),
    "SettlementAllocationIntentCreateParams": (
        "stripe.params.v2.payments._settlement_allocation_intent_create_params",
        False,
    ),
    "SettlementAllocationIntentListParams": (
        "stripe.params.v2.payments._settlement_allocation_intent_list_params",
        False,
    ),
    "SettlementAllocationIntentRetrieveParams": (
        "stripe.params.v2.payments._settlement_allocation_intent_retrieve_params",
        False,
    ),
    "SettlementAllocationIntentSubmitParams": (
        "stripe.params.v2.payments._settlement_allocation_intent_submit_params",
        False,
    ),
    "SettlementAllocationIntentUpdateParams": (
        "stripe.params.v2.payments._settlement_allocation_intent_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired


class GiftCardCreateParams(RequestOptions):
    brand: Literal["fiserv_valuelink", "givex", "svs"]
    """
    The brand of the gift card.
    """
    exp_month: NotRequired[int]
    """
    Two-digit number representing the gift card's expiration month.
    """
    exp_year: NotRequired[int]
    """
    Four-digit number representing the gift card's expiration year.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    number: NotRequired[str]
    """
    The gift card number.
    """
    pin: NotRequired[str]
    """
    The gift card PIN.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.privacy._redaction_job_cancel_params import (
        RedactionJobCancelParams as RedactionJobCancelParams,
    )
    from stripe.params.privacy._redaction_job_create_params import (
        RedactionJobCreateParams as RedactionJobCreateParams,
        RedactionJobCreateParamsObjects as RedactionJobCreateParamsObjects,
    )
    from stripe.params.privacy._redaction_job_list_params import (
        RedactionJobListParams as RedactionJobListParams,
    )
    from stripe.params.privacy._redaction_job_list_validation_errors_params import (
        RedactionJobListValidationErrorsParams as RedactionJobListValidationErrorsParams,
    )
    from stripe.params.privacy._redaction_job_modify_params import (
        RedactionJobModifyParams as RedactionJobModifyParams,
    )
    from stripe.params.privacy._redaction_job_retrieve_params import (
        RedactionJobRetrieveParams as RedactionJobRetrieveParams,
    )
    from stripe.params.privacy._redaction_job_run_params import (
        RedactionJobRunParams as RedactionJobRunParams,
    )
    from stripe.params.privacy._redaction_job_update_params import (
        RedactionJobUpdateParams as RedactionJobUpdateParams,
    )
    from stripe.params.privacy._redaction_job_validate_params import (
        RedactionJobValidateParams as RedactionJobValidateParams,
    )
    from stripe.params.privacy._redaction_job_validation_error_list_params import (
        RedactionJobValidationErrorListParams as RedactionJobValidationErrorListParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "RedactionJobCancelParams": (
        "stripe.params.privacy._redaction_job_cancel_params",
        False,
    ),
    "RedactionJobCreateParams": (
        "stripe.params.privacy._redaction_job_create_params",
        False,
    ),
    "RedactionJobCreateParamsObjects": (
        "stripe.params.privacy._redaction_job_create_params",
        False,
    ),
    "RedactionJobListParams": (
        "stripe.params.privacy._redaction_job_list_params",
        False,
    ),
    "RedactionJobListValidationErrorsParams": (
        "stripe.params.privacy._redaction_job_list_validation_errors_params",
        False,
    ),
    "RedactionJobModifyParams": (
        "stripe.params.privacy._redaction_job_modify_params",
        False,
    ),
    "RedactionJobRetrieveParams": (
        "stripe.params.privacy._redaction_job_retrieve_params",
        False,
    ),
    "RedactionJobRunParams": (
        "stripe.params.privacy._redaction_job_run_params",
        False,
    ),
    "RedactionJobUpdateParams": (
        "stripe.params.privacy._redaction_job_update_params",
        False,
    ),
    "RedactionJobValidateParams": (
        "stripe.params.privacy._redaction_job_validate_params",
        False,
    ),
    "RedactionJobValidationErrorListParams": (
        "stripe.params.privacy._redaction_job_validation_error_list_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar
from typing_extensions import Literal


class FinancialAddressDebitSimulation(StripeObject):
    """
    Debit Simulations represent a simulated debit transaction applied to financial addresses for testing purposes.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.money_management.financial_address_debit_simulation"]
    ] = "v2.money_management.financial_address_debit_simulation"
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.financial_address_debit_simulation"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: str
    """
    The status of the request, signifying whether a simulated debit was initiated.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.money_management._outbound_payment_cancel_params import (
        OutboundPaymentCancelParams,
    )
    from stripe.params.v2.money_management._outbound_payment_create_params import (
        OutboundPaymentCreateParams,
    )
    from stripe.params.v2.money_management._outbound_payment_list_params import (
        OutboundPaymentListParams,
    )
    from stripe.params.v2.money_management._outbound_payment_retrieve_params import (
        OutboundPaymentRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.money_management._outbound_payment import OutboundPayment


class OutboundPaymentService(StripeService):
    def list(
        self,
        params: Optional["OutboundPaymentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OutboundPayment]":
        """
        Returns a list of OutboundPayments that match the provided filters.
        """
        return cast(
            "ListObject[OutboundPayment]",
            self._request(
                "get",
                "/v2/money_management/outbound_payments",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["OutboundPaymentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OutboundPayment]":
        """
        Returns a list of OutboundPayments that match the provided filters.
        """
        return cast(
            "ListObject[OutboundPayment]",
            await self._request_async(
                "get",
                "/v2/money_management/outbound_payments",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "OutboundPaymentCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundPayment":
        """
        Creates an OutboundPayment.
        """
        return cast(
            "OutboundPayment",
            self._request(
                "post",
                "/v2/money_management/outbound_payments",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "OutboundPaymentCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundPayment":
        """
        Creates an OutboundPayment.
        """
        return cast(
            "OutboundPayment",
            await self._request_async(
                "post",
                "/v2/money_management/outbound_payments",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["OutboundPaymentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundPayment":
        """
        Retrieves the details of an existing OutboundPayment by passing the unique OutboundPayment ID from either the OutboundPayment create or list response.
        """
        return cast(
            "OutboundPayment",
            self._request(
                "get",
                "/v2/money_management/outbound_payments/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["OutboundPaymentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundPayment":
        """
        Retrieves the details of an existing OutboundPayment by passing the unique OutboundPayment ID from either the OutboundPayment create or list response.
        """
        return cast(
            "OutboundPayment",
            await self._request_async(
                "get",
                "/v2/money_management/outbound_payments/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        id: str,
        params: Optional["OutboundPaymentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundPayment":
        """
        Cancels an OutboundPayment. Only processing OutboundPayments can be canceled.
        """
        return cast(
            "OutboundPayment",
            self._request(
                "post",
                "/v2/money_management/outbound_payments/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: Optional["OutboundPaymentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundPayment":
        """
        Cancels an OutboundPayment. Only processing OutboundPayments can be canceled.
        """
        return cast(
            "OutboundPayment",
            await self._request_async(
                "post",
                "/v2/money_management/outbound_payments/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class ContractPricingLineQuantityChange(StripeObject):
    """
    A quantity change object for a pricing line, returned by ListContractPricingLineQuantityChanges.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.billing.contract_pricing_line_quantity_change"]
    ] = "v2.billing.contract_pricing_line_quantity_change"

    class Pricing(StripeObject):
        price: Optional[str]
        """
        The ID of the V1 price. Present when `type` is `price`.
        """
        type: Literal["price"]
        """
        The type of pricing.
        """

    created: str
    """
    The timestamp when this quantity change object was created.
    """
    effective_at: str
    """
    The timestamp when this quantity change takes effect.
    """
    id: str
    """
    The ID of the quantity change object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.billing.contract_pricing_line_quantity_change"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    pricing: Pricing
    """
    The pricing configuration for the associated pricing line.
    """
    pricing_line: str
    """
    The ID of the pricing line associated with this quantity change.
    """
    quantity: Decimal
    """
    The quantity at the effective time.
    """
    _inner_class_types = {"pricing": Pricing}
    _field_encodings = {"quantity": "decimal_string"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._expandable_field import ExpandableField
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._discount import Discount
    from stripe._tax_rate import TaxRate
    from stripe.billing._credit_balance_transaction import (
        CreditBalanceTransaction,
    )


class CreditNoteLineItem(StripeObject):
    """
    The credit note line item object
    """

    OBJECT_NAME: ClassVar[Literal["credit_note_line_item"]] = (
        "credit_note_line_item"
    )

    class DiscountAmount(StripeObject):
        amount: int
        """
        The amount, in cents (or local equivalent), of the discount.
        """
        discount: ExpandableField["Discount"]
        """
        The discount that was applied to get this discount amount.
        """

    class PretaxCreditAmount(StripeObject):
        amount: int
        """
        The amount, in cents (or local equivalent), of the pretax credit amount.
        """
        credit_balance_transaction: Optional[
            ExpandableField["CreditBalanceTransaction"]
        ]
        """
        The credit balance transaction that was applied to get this pretax credit amount.
        """
        discount: Optional[ExpandableField["Discount"]]
        """
        The discount that was applied to get this pretax credit amount.
        """
        type: Literal["credit_balance_transaction", "discount"]
        """
        Type of the pretax credit amount referenced.
        """

    class TaxCalculationReference(StripeObject):
        calculation_id: Optional[str]
        """
        The calculation identifier for tax calculation response.
        """
        calculation_item_id: Optional[str]
        """
        The calculation identifier for tax calculation response line item.
        """

    class Tax(StripeObject):
        class TaxRateDetails(StripeObject):
            tax_rate: ExpandableField["TaxRate"]
            """
            ID of the tax rate
            """

        amount: int
        """
        The amount of the tax, in cents (or local equivalent).
        """
        tax_behavior: Literal["exclusive", "inclusive"]
        """
        Whether this tax is inclusive or exclusive.
        """
        tax_rate_details: Optional[TaxRateDetails]
        """
        Additional details about the tax rate. Only present when `type` is `tax_rate_details`.
        """
        taxability_reason: Literal[
            "customer_exempt",
            "not_available",
            "not_collecting",
            "not_subject_to_tax",
            "not_supported",
            "portion_product_exempt",
            "portion_reduced_rated",
            "portion_standard_rated",
            "product_exempt",
            "product_exempt_holiday",
            "proportionally_rated",
            "reduced_rated",
            "reverse_charge",
            "standard_rated",
            "taxable_basis_reduced",
            "zero_rated",
        ]
        """
        The reasoning behind this tax, for example, if the product is tax exempt. The possible values for this field may be extended as new tax rules are supported.
        """
        taxable_amount: Optional[int]
        """
        The amount on which tax is calculated, in cents (or local equivalent).
        """
        type: Literal["tax_rate_details"]
        """
        The type of tax information.
        """
        _inner_class_types = {"tax_rate_details": TaxRateDetails}

    amount: int
    """
    The integer amount in cents (or local equivalent) representing the gross amount being credited for this line item, excluding (exclusive) tax and discounts.
    """
    description: Optional[str]
    """
    Description of the item being credited.
    """
    discount_amount: int
    """
    The integer amount in cents (or local equivalent) representing the discount being credited for this line item.
    """
    discount_amounts: List[DiscountAmount]
    """
    The amount of discount calculated per discount for this line item
    """
    id: str
    """
    Unique identifier for the object.
    """
    invoice_line_item: Optional[str]
    """
    ID of the invoice line item being credited
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["credit_note_line_item"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    pretax_credit_amounts: List[PretaxCreditAmount]
    """
    The pretax credit amounts (ex: discount, credit grants, etc) for this line item.
    """
    quantity: Optional[int]
    """
    The number of units of product being credited.
    """
    tax_calculation_reference: Optional[TaxCalculationReference]
    """
    The tax calculation identifiers of the line item.
    """
    tax_rates: List["TaxRate"]
    """
    The tax rates which apply to the line item.
    """
    taxes: Optional[List[Tax]]
    """
    The tax information of the line item.
    """
    type: Literal["custom_line_item", "invoice_line_item"]
    """
    The type of the credit note line item, one of `invoice_line_item` or `custom_line_item`. When the type is `invoice_line_item` there is an additional `invoice_line_item` property on the resource the value of which is the id of the credited line item on the invoice.
    """
    unit_amount: Optional[int]
    """
    The cost of each unit of product being credited.
    """
    unit_amount_decimal: Optional[Decimal]
    """
    Same as `unit_amount`, but contains a decimal value with at most 12 decimal places.
    """
    _inner_class_types = {
        "discount_amounts": DiscountAmount,
        "pretax_credit_amounts": PretaxCreditAmount,
        "tax_calculation_reference": TaxCalculationReference,
        "taxes": Tax,
    }
    _field_encodings = {"unit_amount_decimal": "decimal_string"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class OutboundPaymentQuote(StripeObject):
    """
    OutboundPaymentQuote represents a quote that provides fee and amount estimates for OutboundPayment.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.money_management.outbound_payment_quote"]
    ] = "v2.money_management.outbound_payment_quote"

    class DeliveryOptions(StripeObject):
        bank_account: Optional[Literal["automatic", "local", "wire"]]
        """
        Open Enum. Method for bank account.
        """
        speed: Optional[Literal["instant", "next_business_day", "standard"]]
        """
        Open Enum. Speed of the payout.
        """

    class EstimatedFee(StripeObject):
        amount: Amount
        """
        The fee amount for corresponding fee type.
        """
        type: Literal[
            "cross_border_payout_fee",
            "foreign_exchange_fee",
            "instant_payout_fee",
            "next_day_payout_fee",
            "real_time_payout_fee",
            "standard_payout_fee",
            "wire_payout_fee",
        ]
        """
        The fee type.
        """

    class From(StripeObject):
        debited: Amount
        """
        The monetary amount debited from the sender, only set on responses.
        """
        financial_account: str
        """
        The FinancialAccount that funds were pulled from.
        """

    class FxQuote(StripeObject):
        class Rates(StripeObject):
            exchange_rate: str
            """
            The exchange rate going from_currency -> to_currency.
            """

        lock_duration: Literal["five_minutes", "none"]
        """
        The duration the exchange rate lock remains valid from creation time. Allowed value is five_minutes or none.
        """
        lock_expires_at: Optional[str]
        """
        Time at which the rate lock will expire, measured in seconds since the Unix epoch. Null when rate locking is not supported.
        """
        lock_status: Literal["active", "expired", "none"]
        """
        Lock status of the quote. Transitions from active to expired once past the lock_expires_at timestamp. Value can be active, expired or none.
        """
        rates: UntypedStripeObject[Rates]
        """
        Key pair: from currency Value: exchange rate going from_currency -> to_currency.
        """
        to_currency: str
        """
        The currency that the transaction is exchanging to.
        """
        _inner_class_types = {"rates": Rates}
        _inner_class_dicts = ["rates"]

    class To(StripeObject):
        credited: Amount
        """
        The monetary amount being credited to the destination.
        """
        payout_method: str
        """
        The payout method which the OutboundPayment uses to send payout.
        """
        recipient: str
        """
        To which account the OutboundPayment is sent.
        """

    amount: Amount
    """
    The "presentment amount" for the OutboundPaymentQuote.
    """
    created: str
    """
    Time at which the OutboundPaymentQuote was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    delivery_options: Optional[DeliveryOptions]
    """
    Delivery options to be used to send the OutboundPayment.
    """
    estimated_fees: List[EstimatedFee]
    """
    The estimated fees for the OutboundPaymentQuote.
    """
    from_: From
    """
    Details about the sender of an OutboundPaymentQuote.
    """
    fx_quote: FxQuote
    """
    The underlying FXQuote details for the OutboundPaymentQuote.
    """
    id: str
    """
    Unique identifier for the OutboundPaymentQuote.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.outbound_payment_quote"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    to: To
    """
    Details about the recipient of an OutboundPaymentQuote.
    """
    _inner_class_types = {
        "delivery_options": DeliveryOptions,
        "estimated_fees": EstimatedFee,
        "from": From,
        "fx_quote": FxQuote,
        "to": To,
    }
    _field_remappings = {"from_": "from"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class PricingPlanSubscriptionRemoveDiscountsParams(TypedDict):
    include: NotRequired[
        List[Literal["discount_details", "pricing_plan_component_details"]]
    ]
    """
    Expand to include additional data such as discount_details.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._payment_attempt_record import PaymentAttemptRecord
    from stripe._request_options import RequestOptions
    from stripe.params._payment_attempt_record_list_params import (
        PaymentAttemptRecordListParams,
    )
    from stripe.params._payment_attempt_record_report_authenticated_params import (
        PaymentAttemptRecordReportAuthenticatedParams,
    )
    from stripe.params._payment_attempt_record_report_authorized_params import (
        PaymentAttemptRecordReportAuthorizedParams,
    )
    from stripe.params._payment_attempt_record_report_canceled_params import (
        PaymentAttemptRecordReportCanceledParams,
    )
    from stripe.params._payment_attempt_record_report_early_fraud_warning_params import (
        PaymentAttemptRecordReportEarlyFraudWarningParams,
    )
    from stripe.params._payment_attempt_record_report_failed_params import (
        PaymentAttemptRecordReportFailedParams,
    )
    from stripe.params._payment_attempt_record_report_guaranteed_params import (
        PaymentAttemptRecordReportGuaranteedParams,
    )
    from stripe.params._payment_attempt_record_report_informational_params import (
        PaymentAttemptRecordReportInformationalParams,
    )
    from stripe.params._payment_attempt_record_report_refund_params import (
        PaymentAttemptRecordReportRefundParams,
    )
    from stripe.params._payment_attempt_record_retrieve_params import (
        PaymentAttemptRecordRetrieveParams,
    )


class PaymentAttemptRecordService(StripeService):
    def list(
        self,
        params: "PaymentAttemptRecordListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentAttemptRecord]":
        """
        List all the Payment Attempt Records attached to the specified Payment Record.
        """
        return cast(
            "ListObject[PaymentAttemptRecord]",
            self._request(
                "get",
                "/v1/payment_attempt_records",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: "PaymentAttemptRecordListParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentAttemptRecord]":
        """
        List all the Payment Attempt Records attached to the specified Payment Record.
        """
        return cast(
            "ListObject[PaymentAttemptRecord]",
            await self._request_async(
                "get",
                "/v1/payment_attempt_records",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["PaymentAttemptRecordRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Retrieves a Payment Attempt Record with the given ID
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "get",
                "/v1/payment_attempt_records/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["PaymentAttemptRecordRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Retrieves a Payment Attempt Record with the given ID
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "get",
                "/v1/payment_attempt_records/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_authenticated(
        self,
        payment_attempt_record: str,
        params: Optional[
            "PaymentAttemptRecordReportAuthenticatedParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was authenticated.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_authenticated".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_authenticated_async(
        self,
        payment_attempt_record: str,
        params: Optional[
            "PaymentAttemptRecordReportAuthenticatedParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was authenticated.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_authenticated".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_authorized(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportAuthorizedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was authorized.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_authorized".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_authorized_async(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportAuthorizedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was authorized.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_authorized".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_canceled(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportCanceledParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was canceled.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_canceled".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_canceled_async(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportCanceledParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was canceled.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_canceled".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_early_fraud_warning(
        self,
        payment_attempt_record: str,
        params: "PaymentAttemptRecordReportEarlyFraudWarningParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record received an early fraud warning.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_early_fraud_warning".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_early_fraud_warning_async(
        self,
        payment_attempt_record: str,
        params: "PaymentAttemptRecordReportEarlyFraudWarningParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record received an early fraud warning.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_early_fraud_warning".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_failed(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportFailedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record failed.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_failed".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_failed_async(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportFailedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record failed.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_failed".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_guaranteed(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportGuaranteedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was guaranteed.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_guaranteed".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_guaranteed_async(
        self,
        payment_attempt_record: str,
        params: Optional["PaymentAttemptRecordReportGuaranteedParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was guaranteed.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_guaranteed".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_informational(
        self,
        payment_attempt_record: str,
        params: Optional[
            "PaymentAttemptRecordReportInformationalParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report informational updates on the specified Payment Attempt Record.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_informational".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_informational_async(
        self,
        payment_attempt_record: str,
        params: Optional[
            "PaymentAttemptRecordReportInformationalParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report informational updates on the specified Payment Attempt Record.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_informational".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def report_refund(
        self,
        payment_attempt_record: str,
        params: "PaymentAttemptRecordReportRefundParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was refunded.
        """
        return cast(
            "PaymentAttemptRecord",
            self._request(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_refund".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def report_refund_async(
        self,
        payment_attempt_record: str,
        params: "PaymentAttemptRecordReportRefundParams",
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentAttemptRecord":
        """
        Report that the specified Payment Attempt Record was refunded.
        """
        return cast(
            "PaymentAttemptRecord",
            await self._request_async(
                "post",
                "/v1/payment_attempt_records/{payment_attempt_record}/report_refund".format(
                    payment_attempt_record=sanitize_id(payment_attempt_record),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.shared_payment._issued_token_create_params import (
        IssuedTokenCreateParams,
    )
    from stripe.params.shared_payment._issued_token_retrieve_params import (
        IssuedTokenRetrieveParams,
    )
    from stripe.params.shared_payment._issued_token_revoke_params import (
        IssuedTokenRevokeParams,
    )


class IssuedToken(CreateableAPIResource["IssuedToken"]):
    """
    A SharedPaymentIssuedToken is a limited-use reference to a PaymentMethod that can be created with a secret key. When shared with another Stripe account (Seller), it enables that account to either process a payment on Stripe against a PaymentMethod that your Stripe account owns, or to forward a usable credential created against the originalPaymentMethod to then process the payment off-Stripe.
    """

    OBJECT_NAME: ClassVar[Literal["shared_payment.issued_token"]] = (
        "shared_payment.issued_token"
    )

    class NextAction(StripeObject):
        class UseStripeSdk(StripeObject):
            value: str
            """
            A base64-encoded string used by Stripe.js and the iOS and Android client SDKs to handle the next action. Its content is subject to change.
            """

        type: Literal["use_stripe_sdk"]
        """
        Specifies the type of next action required. Determines which child attribute contains action details.
        """
        use_stripe_sdk: Optional[UseStripeSdk]
        """
        Contains details for handling the next action using Stripe.js, iOS, or Android SDKs. Present when `next_action.type` is `use_stripe_sdk`.
        """
        _inner_class_types = {"use_stripe_sdk": UseStripeSdk}

    class RiskDetails(StripeObject):
        class Insights(StripeObject):
            class Bot(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: float
                """
                Risk score for this insight.
                """

            class CardIssuerDecline(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: float
                """
                Risk score for this insight.
                """

            class CardTesting(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: float
                """
                Risk score for this insight.
                """

            class FraudulentDispute(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: int
                """
                Risk score for this insight.
                """

            class StolenCard(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: int
                """
                Risk score for this insight.
                """

            bot: Optional[Bot]
            """
            Bot risk insight.
            """
            card_issuer_decline: Optional[CardIssuerDecline]
            """
            Card issuer decline risk insight.
            """
            card_testing: Optional[CardTesting]
            """
            Card testing risk insight.
            """
            fraudulent_dispute: Optional[FraudulentDispute]
            """
            Fraudulent dispute risk insight.
            """
            stolen_card: Optional[StolenCard]
            """
            Stolen card risk insight.
            """
            _inner_class_types = {
                "bot": Bot,
                "card_issuer_decline": CardIssuerDecline,
                "card_testing": CardTesting,
                "fraudulent_dispute": FraudulentDispute,
                "stolen_card": StolenCard,
            }

        insights: Insights
        """
        Risk insights for this token, including scores and recommended actions for each risk type.
        """
        _inner_class_types = {"insights": Insights}

    class SellerDetails(StripeObject):
        external_id: str
        """
        A unique id within a network that identifies a logical seller. This should usually be the merchant id on the seller platform.
        """
        network_business_profile: str
        """
        The unique and logical string that identifies the seller platform that this SharedToken is being created for.
        """

    class UsageDetails(StripeObject):
        class AmountCaptured(StripeObject):
            currency: str
            """
            Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """
            value: int
            """
            Integer value of the amount in the smallest currency unit.
            """

        amount_captured: Optional[AmountCaptured]
        """
        The total amount captured using this SharedPaymentToken.
        """
        _inner_class_types = {"amount_captured": AmountCaptured}

    class UsageLimits(StripeObject):
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        expires_at: Optional[int]
        """
        Time at which this SharedPaymentToken expires and can no longer be used to confirm a PaymentIntent.
        """
        max_amount: int
        """
        Max amount that can be captured using this SharedPaymentToken.
        """
        recurring_interval: Optional[Literal["month", "week", "year"]]
        """
        The recurring interval at which the shared payment token's amount usage restrictions reset.
        """

    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deactivated_at: Optional[int]
    """
    Time at which this SharedPaymentIssuedToken was deactivated.
    """
    deactivated_reason: Optional[
        Literal["consumed", "expired", "resolved", "revoked"]
    ]
    """
    The reason why the SharedPaymentIssuedToken has been deactivated.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    next_action: Optional[NextAction]
    """
    If present, describes the action required to make this `SharedPaymentIssuedToken` usable for payments. Present when the token is in `requires_action` state.
    """
    object: Literal["shared_payment.issued_token"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    payment_method: str
    """
    ID of an existing PaymentMethod.
    """
    return_url: Optional[str]
    """
    If the customer does not exit their browser while authenticating, they will be redirected to this specified URL after completion.
    """
    risk_details: Optional[RiskDetails]
    """
    Risk details of the SharedPaymentIssuedToken.
    """
    seller_details: Optional[SellerDetails]
    """
    Seller details of the SharedPaymentIssuedToken, including network_id and external_id.
    """
    setup_future_usage: Optional[Literal["on_session"]]
    """
    Indicates that you intend to save the PaymentMethod of this SharedPaymentToken to a customer later.
    """
    shared_metadata: Optional[UntypedStripeObject[str]]
    """
    Metadata about the SharedPaymentIssuedToken.
    """
    status: Optional[Literal["active", "deactivated", "requires_action"]]
    """
    Status of this SharedPaymentIssuedToken, one of `active`, `requires_action`, or `deactivated`.
    """
    usage_details: Optional[UsageDetails]
    """
    Usage details of the SharedPaymentIssuedToken
    """
    usage_limits: Optional[UsageLimits]
    """
    Usage limits of the SharedPaymentIssuedToken.
    """

    @classmethod
    def create(
        cls, **params: Unpack["IssuedTokenCreateParams"]
    ) -> "IssuedToken":
        """
        Creates a new SharedPaymentIssuedToken object
        """
        return cast(
            "IssuedToken",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["IssuedTokenCreateParams"]
    ) -> "IssuedToken":
        """
        Creates a new SharedPaymentIssuedToken object
        """
        return cast(
            "IssuedToken",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["IssuedTokenRetrieveParams"]
    ) -> "IssuedToken":
        """
        Retrieves an existing SharedPaymentIssuedToken object
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["IssuedTokenRetrieveParams"]
    ) -> "IssuedToken":
        """
        Retrieves an existing SharedPaymentIssuedToken object
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_revoke(
        cls,
        shared_payment_issued_token: str,
        **params: Unpack["IssuedTokenRevokeParams"],
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        return cast(
            "IssuedToken",
            cls._static_request(
                "post",
                "/v1/shared_payment/issued_tokens/{shared_payment_issued_token}/revoke".format(
                    shared_payment_issued_token=sanitize_id(
                        shared_payment_issued_token
                    )
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def revoke(
        shared_payment_issued_token: str,
        **params: Unpack["IssuedTokenRevokeParams"],
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        ...

    @overload
    def revoke(
        self, **params: Unpack["IssuedTokenRevokeParams"]
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        ...

    @class_method_variant("_cls_revoke")
    def revoke(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["IssuedTokenRevokeParams"]
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        return cast(
            "IssuedToken",
            self._request(
                "post",
                "/v1/shared_payment/issued_tokens/{shared_payment_issued_token}/revoke".format(
                    shared_payment_issued_token=sanitize_id(
                        self._data.get("id")
                    )
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_revoke_async(
        cls,
        shared_payment_issued_token: str,
        **params: Unpack["IssuedTokenRevokeParams"],
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        return cast(
            "IssuedToken",
            await cls._static_request_async(
                "post",
                "/v1/shared_payment/issued_tokens/{shared_payment_issued_token}/revoke".format(
                    shared_payment_issued_token=sanitize_id(
                        shared_payment_issued_token
                    )
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def revoke_async(
        shared_payment_issued_token: str,
        **params: Unpack["IssuedTokenRevokeParams"],
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        ...

    @overload
    async def revoke_async(
        self, **params: Unpack["IssuedTokenRevokeParams"]
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        ...

    @class_method_variant("_cls_revoke_async")
    async def revoke_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["IssuedTokenRevokeParams"]
    ) -> "IssuedToken":
        """
        Revokes a SharedPaymentIssuedToken
        """
        return cast(
            "IssuedToken",
            await self._request_async(
                "post",
                "/v1/shared_payment/issued_tokens/{shared_payment_issued_token}/revoke".format(
                    shared_payment_issued_token=sanitize_id(
                        self._data.get("id")
                    )
                ),
                params=params,
            ),
        )

    _inner_class_types = {
        "next_action": NextAction,
        "risk_details": RiskDetails,
        "seller_details": SellerDetails,
        "usage_details": UsageDetails,
        "usage_limits": UsageLimits,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired


class WebhookEndpointCreateParams(RequestOptions):
    api_version: NotRequired[
        Literal[
            "2011-01-01",
            "2011-06-21",
            "2011-06-28",
            "2011-08-01",
            "2011-09-15",
            "2011-11-17",
            "2012-02-23",
            "2012-03-25",
            "2012-06-18",
            "2012-06-28",
            "2012-07-09",
            "2012-09-24",
            "2012-10-26",
            "2012-11-07",
            "2013-02-11",
            "2013-02-13",
            "2013-07-05",
            "2013-08-12",
            "2013-08-13",
            "2013-10-29",
            "2013-12-03",
            "2014-01-31",
            "2014-03-13",
            "2014-03-28",
            "2014-05-19",
            "2014-06-13",
            "2014-06-17",
            "2014-07-22",
            "2014-07-26",
            "2014-08-04",
            "2014-08-20",
            "2014-09-08",
            "2014-10-07",
            "2014-11-05",
            "2014-11-20",
            "2014-12-08",
            "2014-12-17",
            "2014-12-22",
            "2015-01-11",
            "2015-01-26",
            "2015-02-10",
            "2015-02-16",
            "2015-02-18",
            "2015-03-24",
            "2015-04-07",
            "2015-06-15",
            "2015-07-07",
            "2015-07-13",
            "2015-07-28",
            "2015-08-07",
            "2015-08-19",
            "2015-09-03",
            "2015-09-08",
            "2015-09-23",
            "2015-10-01",
            "2015-10-12",
            "2015-10-16",
            "2016-02-03",
            "2016-02-19",
            "2016-02-22",
            "2016-02-23",
            "2016-02-29",
            "2016-03-07",
            "2016-06-15",
            "2016-07-06",
            "2016-10-19",
            "2017-01-27",
            "2017-02-14",
            "2017-04-06",
            "2017-05-25",
            "2017-06-05",
            "2017-08-15",
            "2017-12-14",
            "2018-01-23",
            "2018-02-05",
            "2018-02-06",
            "2018-02-28",
            "2018-05-21",
            "2018-07-27",
            "2018-08-23",
            "2018-09-06",
            "2018-09-24",
            "2018-10-31",
            "2018-11-08",
            "2019-02-11",
            "2019-02-19",
            "2019-03-14",
            "2019-05-16",
            "2019-08-14",
            "2019-09-09",
            "2019-10-08",
            "2019-10-17",
            "2019-11-05",
            "2019-12-03",
            "2020-03-02",
            "2020-08-27",
            "2022-08-01",
            "2022-11-15",
            "2023-08-16",
            "2023-10-16",
            "2024-04-10",
            "2024-06-20",
            "2024-09-30.acacia",
            "2024-10-28.acacia",
            "2024-11-20.acacia",
            "2024-12-18.acacia",
            "2025-01-27.acacia",
            "2025-02-24.acacia",
            "2025-03-01.dashboard",
            "2025-03-31.basil",
            "2025-04-30.basil",
            "2025-05-28.basil",
            "2025-06-30.basil",
            "2025-07-30.basil",
            "2025-08-27.basil",
            "2025-09-30.clover",
            "2025-10-29.clover",
            "2025-11-17.clover",
            "2025-12-15.clover",
            "2026-01-28.clover",
            "2026-02-25.clover",
            "2026-03-25.dahlia",
            "2026-04-22.dahlia",
            "2026-05-27.dahlia",
            "2026-06-24.dahlia",
        ]
    ]
    """
    Events sent to this endpoint will be generated with this Stripe Version instead of your account's default Stripe Version.
    """
    connect: NotRequired[bool]
    """
    Whether this endpoint should receive events from connected accounts (`true`), or from your account (`false`). Defaults to `false`.
    """
    description: NotRequired["Literal['']|str"]
    """
    An optional description of what the webhook is used for.
    """
    enabled_events: List[
        Literal[
            "*",
            "account.application.authorized",
            "account.application.deauthorized",
            "account.external_account.created",
            "account.external_account.deleted",
            "account.external_account.updated",
            "account.updated",
            "account_notice.created",
            "account_notice.updated",
            "application_fee.created",
            "application_fee.refund.updated",
            "application_fee.refunded",
            "balance.available",
            "balance_settings.updated",
            "billing.alert.triggered",
            "billing.credit_balance_transaction.created",
            "billing.credit_grant.created",
            "billing.credit_grant.updated",
            "billing.meter.created",
            "billing.meter.deactivated",
            "billing.meter.reactivated",
            "billing.meter.updated",
            "billing_portal.configuration.created",
            "billing_portal.configuration.updated",
            "billing_portal.session.created",
            "capability.updated",
            "capital.financing_offer.accepted",
            "capital.financing_offer.accepted_other_offer",
            "capital.financing_offer.canceled",
            "capital.financing_offer.created",
            "capital.financing_offer.expired",
            "capital.financing_offer.fully_repaid",
            "capital.financing_offer.paid_out",
            "capital.financing_offer.rejected",
            "capital.financing_offer.replacement_created",
            "capital.financing_summary.line_of_credit_update",
            "capital.financing_transaction.created",
            "cash_balance.funds_available",
            "charge.captured",
            "charge.dispute.closed",
            "charge.dispute.created",
            "charge.dispute.funds_reinstated",
            "charge.dispute.funds_withdrawn",
            "charge.dispute.updated",
            "charge.expired",
            "charge.failed",
            "charge.pending",
            "charge.refund.updated",
            "charge.refunded",
            "charge.succeeded",
            "charge.updated",
            "checkout.session.async_payment_failed",
            "checkout.session.async_payment_succeeded",
            "checkout.session.completed",
            "checkout.session.expired",
            "climate.order.canceled",
            "climate.order.created",
            "climate.order.delayed",
            "climate.order.delivered",
            "climate.order.product_substituted",
            "climate.product.created",
            "climate.product.pricing_updated",
            "coupon.created",
            "coupon.deleted",
            "coupon.updated",
            "credit_note.created",
            "credit_note.updated",
            "credit_note.voided",
            "customer.created",
            "customer.deleted",
            "customer.discount.created",
            "customer.discount.deleted",
            "customer.discount.updated",
            "customer.source.created",
            "customer.source.deleted",
            "customer.source.expiring",
            "customer.source.updated",
            "customer.subscription.collection_paused",
            "customer.subscription.collection_resumed",
            "customer.subscription.created",
            "customer.subscription.custom_event",
            "customer.subscription.deleted",
            "customer.subscription.paused",
            "customer.subscription.pending_update_applied",
            "customer.subscription.pending_update_expired",
            "customer.subscription.price_migration_failed",
            "customer.subscription.resumed",
            "customer.subscription.trial_will_end",
            "customer.subscription.updated",
            "customer.tax_id.created",
            "customer.tax_id.deleted",
            "customer.tax_id.updated",
            "customer.updated",
            "customer_cash_balance_transaction.created",
            "entitlements.active_entitlement_summary.updated",
            "file.created",
            "financial_connections.account.account_numbers_updated",
            "financial_connections.account.created",
            "financial_connections.account.deactivated",
            "financial_connections.account.disconnected",
            "financial_connections.account.reactivated",
            "financial_connections.account.refreshed_balance",
            "financial_connections.account.refreshed_inferred_balances",
            "financial_connections.account.refreshed_ownership",
            "financial_connections.account.refreshed_transactions",
            "financial_connections.account.upcoming_account_number_expiry",
            "financial_connections.session.updated",
            "fx_quote.expired",
            "identity.verification_session.canceled",
            "identity.verification_session.created",
            "identity.verification_session.processing",
            "identity.verification_session.redacted",
            "identity.verification_session.requires_input",
            "identity.verification_session.verified",
            "invoice.created",
            "invoice.deleted",
            "invoice.finalization_failed",
            "invoice.finalized",
            "invoice.marked_uncollectible",
            "invoice.overdue",
            "invoice.overpaid",
            "invoice.paid",
            "invoice.payment.overpaid",
            "invoice.payment_action_required",
            "invoice.payment_attempt_required",
            "invoice.payment_failed",
            "invoice.payment_succeeded",
            "invoice.sent",
            "invoice.upcoming",
            "invoice.updated",
            "invoice.voided",
            "invoice.will_be_due",
            "invoice_payment.paid",
            "invoiceitem.created",
            "invoiceitem.deleted",
            "issuing_authorization.created",
            "issuing_authorization.request",
            "issuing_authorization.updated",
            "issuing_card.created",
            "issuing_card.updated",
            "issuing_cardholder.created",
            "issuing_cardholder.updated",
            "issuing_dispute.closed",
            "issuing_dispute.created",
            "issuing_dispute.funds_reinstated",
            "issuing_dispute.funds_rescinded",
            "issuing_dispute.submitted",
            "issuing_dispute.updated",
            "issuing_dispute_settlement_detail.created",
            "issuing_dispute_settlement_detail.updated",
            "issuing_fraud_liability_debit.created",
            "issuing_personalization_design.activated",
            "issuing_personalization_design.deactivated",
            "issuing_personalization_design.rejected",
            "issuing_personalization_design.updated",
            "issuing_settlement.created",
            "issuing_settlement.updated",
            "issuing_token.created",
            "issuing_token.updated",
            "issuing_transaction.created",
            "issuing_transaction.purchase_details_receipt_updated",
            "issuing_transaction.updated",
            "mandate.updated",
            "payment_intent.amount_capturable_updated",
            "payment_intent.canceled",
            "payment_intent.created",
            "payment_intent.partially_funded",
            "payment_intent.payment_failed",
            "payment_intent.processing",
            "payment_intent.requires_action",
            "payment_intent.succeeded",
            "payment_link.created",
            "payment_link.updated",
            "payment_method.attached",
            "payment_method.automatically_updated",
            "payment_method.detached",
            "payment_method.updated",
            "payout.canceled",
            "payout.created",
            "payout.failed",
            "payout.paid",
            "payout.reconciliation_completed",
            "payout.updated",
            "person.created",
            "person.deleted",
            "person.updated",
            "plan.created",
            "plan.deleted",
            "plan.updated",
            "price.created",
            "price.deleted",
            "price.updated",
            "privacy.redaction_job.canceled",
            "privacy.redaction_job.created",
            "privacy.redaction_job.ready",
            "privacy.redaction_job.succeeded",
            "privacy.redaction_job.validation_error",
            "product.created",
            "product.deleted",
            "product.updated",
            "promotion_code.created",
            "promotion_code.updated",
            "quote.accept_failed",
            "quote.accepted",
            "quote.accepting",
            "quote.canceled",
            "quote.created",
            "quote.draft",
            "quote.finalized",
            "quote.reestimate_failed",
            "quote.reestimated",
            "quote.stale",
            "radar.early_fraud_warning.created",
            "radar.early_fraud_warning.updated",
            "refund.created",
            "refund.failed",
            "refund.updated",
            "reporting.report_run.failed",
            "reporting.report_run.succeeded",
            "reporting.report_type.updated",
            "reserve.hold.created",
            "reserve.hold.updated",
            "reserve.plan.created",
            "reserve.plan.disabled",
            "reserve.plan.expired",
            "reserve.plan.updated",
            "reserve.release.created",
            "review.closed",
            "review.opened",
            "setup_intent.canceled",
            "setup_intent.created",
            "setup_intent.requires_action",
            "setup_intent.setup_failed",
            "setup_intent.succeeded",
            "sigma.scheduled_query_run.created",
            "source.canceled",
            "source.chargeable",
            "source.failed",
            "source.mandate_notification",
            "source.refund_attributes_required",
            "source.transaction.created",
            "source.transaction.updated",
            "subscription_schedule.aborted",
            "subscription_schedule.canceled",
            "subscription_schedule.completed",
            "subscription_schedule.created",
            "subscription_schedule.expiring",
            "subscription_schedule.price_migration_failed",
            "subscription_schedule.released",
            "subscription_schedule.updated",
            "tax.form.updated",
            "tax.settings.updated",
            "tax_rate.created",
            "tax_rate.updated",
            "terminal.reader.action_failed",
            "terminal.reader.action_succeeded",
            "terminal.reader.action_updated",
            "test_helpers.test_clock.advancing",
            "test_helpers.test_clock.created",
            "test_helpers.test_clock.deleted",
            "test_helpers.test_clock.internal_failure",
            "test_helpers.test_clock.ready",
            "topup.canceled",
            "topup.created",
            "topup.failed",
            "topup.reversed",
            "topup.succeeded",
            "transfer.created",
            "transfer.reversed",
            "transfer.updated",
            "treasury.credit_reversal.created",
            "treasury.credit_reversal.posted",
            "treasury.debit_reversal.completed",
            "treasury.debit_reversal.created",
            "treasury.debit_reversal.initial_credit_granted",
            "treasury.financial_account.closed",
            "treasury.financial_account.created",
            "treasury.financial_account.features_status_updated",
            "treasury.inbound_transfer.canceled",
            "treasury.inbound_transfer.created",
            "treasury.inbound_transfer.failed",
            "treasury.inbound_transfer.succeeded",
            "treasury.outbound_payment.canceled",
            "treasury.outbound_payment.created",
            "treasury.outbound_payment.expected_arrival_date_updated",
            "treasury.outbound_payment.failed",
            "treasury.outbound_payment.posted",
            "treasury.outbound_payment.returned",
            "treasury.outbound_payment.tracking_details_updated",
            "treasury.outbound_transfer.canceled",
            "treasury.outbound_transfer.created",
            "treasury.outbound_transfer.expected_arrival_date_updated",
            "treasury.outbound_transfer.failed",
            "treasury.outbound_transfer.posted",
            "treasury.outbound_transfer.returned",
            "treasury.outbound_transfer.tracking_details_updated",
            "treasury.received_credit.created",
            "treasury.received_credit.failed",
            "treasury.received_credit.succeeded",
            "treasury.received_debit.created",
        ]
    ]
    """
    The list of events to enable for this endpoint. You may specify `['*']` to enable all events, except those that require explicit selection.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    url: str
    """
    The URL of the webhook endpoint.
    """
